/*
 * Copyright (c) 2009 Yoshikazu Kuramochi
 * All rights reserved.
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */

package ch.kuramo.javie.core;

import java.util.Collection;
import java.util.Set;

import org.mozilla.javascript.Context;
import org.mozilla.javascript.ScriptRuntime;
import org.mozilla.javascript.Scriptable;

import ch.kuramo.javie.api.plugin.PIAnimatableInteger;
import ch.kuramo.javie.core.exprelems.IntegerProperty;

public class AnimatableInteger extends AbstractAnimatableValue<Integer> implements PIAnimatableInteger {

	private static final Set<Interpolation> SUPPORTED_INTERPOLATIONS = Util.unmodifiableSet(
			Interpolation.HOLD, Interpolation.LINEAR, Interpolation.CATMULL_ROM);


	public AnimatableInteger(Integer staticValue, Collection<Keyframe<Integer>> keyframes, String expression) {
		super(staticValue, keyframes, expression);
	}

	public AnimatableInteger(Integer staticValue) {
		super(staticValue);
	}

	public Integer jsToJava(Object jsValue) {
		return (Integer) Context.jsToJava(jsValue, int.class);
	}

	@Override
	protected double[] toArray(Integer value) {
		return new double[] { value.doubleValue() };
	}

	@Override
	protected Integer valueOf(double[] d) {
		return Integer.valueOf(Math.round((float) d[0]));
	}

	@Override
	public Set<Interpolation> supportedInterpolations() {
		return SUPPORTED_INTERPOLATIONS;
	}

	public Scriptable createExpressionElement(final RenderContext renderContext) {
		IntegerProperty ip = new IntegerProperty() {
			public int getValue()				{ return value(renderContext); }
			public int valueAtTime(double t)	{ return AnimatableInteger.this.valueAtTime(t, renderContext); }
		};
		return renderContext.toNativeJavaObject(ip, ScriptRuntime.NumberClass);
	}

}
